# The example of use of nnet function of R applied to time series is based on 
# the World Exports of Fuels, US $ at current prices
# source http://stat.wto.org
# finding a series and downloading it as a csv file (see below)

# loading functions used below (the file has to be in the same folder)
from myFunctions import *

# to plot from Python
import matplotlib.pyplot as plt
# matplotlib tutorial at http://matplotlib.org/users/pyplot_tutorial.html
# installation sources at http://matplotlib.org/downloads.html

# retrieveing the data from the file TsExport5795186.csv
# the path /home/pt/Desktop/series/ has to be modified following each local
# situation

# e.g.
f=open("./TsExport5795186.csv","r")
#f=open("/Users/pt/Desktop/timeSeriesNNs/TsExport5795186.csv","r")

# the following one is a possible way to read the data from a text file (a csv
# in our case; csv = comma separated values; in some cases, instead of ','
# we have other signs, as semicolon or others ...)

years=[] # the dates (years) will not be used, but anyway ...
fuels=[]

# we extract here the first columns (years) and
# the second one (world export of fuels in $ at current prices)

for line in f: #reading by lines
    line=line.replace(","," ").split()
    years.append(line[0])
    fuels.append(float(line[1]))

f.close    
    
# using matplotlib we plot the fuel series

# looking at the plot we see that the final part is quite complicated; instead 
# the segment 
fuelsShort=fuels[0:19]
# is a lot simpler

fig=plt.figure()
plt.ylabel('world export of fuels')
plt.xlabel('1990-2011                                      1990-2008')

fig1=fig.add_subplot(121)
# 121 means 1 row, 2 cols, position 1 in the window
fig1.plot(fuels)

fig2=fig.add_subplot(122)
fig2.plot(fuelsShort)

plt.show()


# prepare the data for neural network estimation, with a window moving on the 
# series
#
# being the series x0, x1, x3, x3, ..., xn
# a moving window of (e.g.) length 4 would produce a table with
#
# independent (input) var.         dependent (output) variable
# x0      x1      x2                 x3
# x1      x2      x3                 x4
# x2      x3      x4                 x5
# ... 
# xn-3    xn-2    xn-1               xn
#
# we could also use more complicated tables with intervals in the rows as in the case
# x0, x5, x10 etc. 


# window size (input of length ws-1)
ws=5
dataIn=[]
dataOu=[]
makeWindow(ws,fuels,dataIn, dataOu)
dataShortIn=[]
dataShortOu=[]
makeWindow(ws,fuelsShort,dataShortIn,dataShortOu)

# using R (instructions to connect R and Python at 
# http://terna.to.it/econophysics16/cmap/pyRserve_Rserve%20(difficult).pdf
import pyRserve
cn=pyRserve.connect(host="localhost")

# load nnet in R (usually is already in R; if necessary run once in R
# install.packages('nnet')
cn.r('library(nnet)')

# alternatively we could adapt the program (mainly the function 
# estimateNN_and_predict() to the use of the library 'neuralnet'
# look at http://cran.r-project.org/web/packages/neuralnet/neuralnet.pdf
# and http://cran.r-project.org/web/packages/neuralnet/index.html


# make estimates using a subset of data (non considenring in NN estimate the
# final nForecast (nF) rows)
nF=3
# number of hidden nodes (neurons): a simple choice is to state this number 
# equal to the number of input variables (nodes)
hN=ws-1

estimateNN_and_predict(plt,cn,"All the years",ws,hN,nF,dataIn,dataOu)
raw_input("enter to continue")
cn.r('dev.off()')

estimateNN_and_predict(plt,cn,"Initial part of the years",ws,hN,nF,dataShortIn,dataShortOu)
raw_input("enter to continue")
cn.r('dev.off()')

# returns, defined as 
#             xn
#        r = ----  - 1
#            xn-1 


rFuels=[]
for i in range(1,len(fuels)):
    rFuels.append(fuels[i]/fuels[i-1] - 1.)
#print rFuels
rFuelsShort=rFuels[0:18]
#print rFuelsShort


# window size (input of length ws-1)
ws=5
rDataIn=[]
rDataOu=[]
makeWindow(ws,rFuels,rDataIn, rDataOu)
rDataShortIn=[]
rDataShortOu=[]
makeWindow(ws,rFuelsShort,rDataShortIn,rDataShortOu)


estimateNN_and_predict(plt,cn,"All the years (returns)",ws,hN,nF,rDataIn,rDataOu)
raw_input("enter to continue")
cn.r('dev.off()')

estimateNN_and_predict(plt,cn,"Initial part of the years (returns)",ws,hN,nF,rDataShortIn,rDataShortOu)
raw_input("enter to finish")

cn.r('dev.off()')
cn.close()
